package cn.minbb.hid;

import android.content.BroadcastReceiver;
import android.content.Context;
import android.content.Intent;
import android.content.IntentFilter;
import android.hardware.usb.UsbDevice;
import android.hardware.usb.UsbDeviceConnection;
import android.hardware.usb.UsbEndpoint;
import android.hardware.usb.UsbInterface;
import android.hardware.usb.UsbManager;
import android.os.Bundle;
import android.support.v7.app.AppCompatActivity;
import android.util.Log;
import android.view.View;
import android.widget.TextView;
import android.widget.Toast;

import com.felhr.usbserial.UsbSerialDevice;
import com.felhr.usbserial.UsbSerialInterface;
import com.sensetime.serialport.SerialPortHelper;

import java.text.SimpleDateFormat;
import java.util.Arrays;
import java.util.Date;
import java.util.HashMap;

import cn.minbb.hid.services.DataService;

public class MainActivity extends AppCompatActivity {

    private static final String TAG = "MainActivity";

    private UsbManager usbManager;
    private UsbDevice usbDevice;
    private UsbInterface usbInterface;
    private UsbDeviceConnection usbDeviceConnection;
    private UsbEndpoint epIn, epOut;
    private final int VendorID = 1061, ProductID = 33113;

    private HIDDataReceiver receiver;
    private SerialPortHelper mSerialPortHelper;
    private TextView text;
    private Thread threadDataIn;
    private boolean isInterrupted = false;

    @Override
    protected void onCreate(Bundle savedInstanceState) {
        super.onCreate(savedInstanceState);
        setContentView(R.layout.activity_main);

        text = findViewById(R.id.text);

        // 获取UsbManager
        usbManager = (UsbManager) getSystemService(USB_SERVICE);
        enumerateDevice();
        findInterface();
        openDevice();
        assignEndpoint();

        /**
         * 如果你的程序经过上面的四步运行得到下面的打印信息，说明你可以进行通讯处理了
         * 05-27 14:54:24.140: D/USB_HOST(10870): DeviceInfo: 8457 , 30264
         * 05-27 14:54:24.140: D/USB_HOST(10870): 枚举设备成功
         * 05-27 14:54:24.140: D/USB_HOST(10870): interfaceCounts : 1
         * 05-27 14:54:24.140: D/USB_HOST(10870): 找到我的设备接口
         * 05-27 14:54:24.160: D/USB_HOST(10870): 打开设备成功
         * 05-27 14:54:24.170: D/USB_HOST(10870): 到此为止：
         * 发现设备 -> 枚举设备 -> 找到设备的接口 -> 连接设备 -> 分配相应的端点，都已完成，下一步可以进行通讯处理。
         */

//        mSerialPortHelper = SerialPortHelper.newInstance(this);
//        mSerialPortHelper.init();
//        mSerialPortHelper.setCallBack(new SerialPortHelper.CallBack() {
//            @Override
//            public void receivedStartMessage() {
//                Log.d(TAG, "启动人脸比对Activity");
//                // startSenseTimeActivityFromLib();
//            }
//
//            @Override
//            public void onError(int code, String msg) {
//                Log.e("zzf", msg);
//            }
//        });
//
//        mSerialPortHelper.start();

        receiver = new HIDDataReceiver();
        IntentFilter filter = new IntentFilter();
        filter.addAction("cn.minbb.hid.services.DataService");
        MainActivity.this.registerReceiver(receiver, filter);

        UsbSerialDevice serial = UsbSerialDevice.createUsbSerialDevice(usbDevice, usbDeviceConnection);
        if (null != serial) {
            serial.open();
            serial.setBaudRate(115200);
            serial.setDataBits(UsbSerialInterface.DATA_BITS_8);
            serial.setParity(UsbSerialInterface.PARITY_ODD);
            serial.setFlowControl(UsbSerialInterface.FLOW_CONTROL_OFF);
            serial.read(data -> {
                Log.d(TAG, "************收到数据************");
                Log.d(TAG, new String(data));
                text.setText(new String(data));
                Log.d(TAG, "**********************************");
            });
            serial.getCTS(new UsbSerialInterface.UsbCTSCallback() {
                @Override
                public void onCTSChanged(boolean state) {
                    Log.d(TAG, "***************************** " + state);
                    text.setText("*************" + state);
                }
            });
        } else {
            Log.e(TAG, "串口为空");
        }
    }

    @Override
    protected void onDestroy() {
        super.onDestroy();
        if (null != receiver) unregisterReceiver(receiver);
//        mSerialPortHelper.release(); //释放串口
    }

    public class HIDDataReceiver extends BroadcastReceiver {
        @Override
        public void onReceive(Context context, Intent intent) {
            String time = SimpleDateFormat.getTimeInstance().format(new Date());
            Bundle bundle = intent.getExtras();
            if (null != bundle) {
                boolean success = bundle.getBoolean("success");
                if (success) {
                    text.setText(time + ": " + new String(bundle.getByteArray("data")));
                } else {
                    text.setText(time + ": " + bundle.getInt("data"));
                }
            } else {
                Toast.makeText(getApplicationContext(), "数据包为空", Toast.LENGTH_SHORT).show();
            }
        }
    }

    /**
     * 枚举设备
     */
    private void enumerateDevice() {
        if (usbManager == null) return;
        HashMap<String, UsbDevice> deviceList = usbManager.getDeviceList();
        if (!deviceList.isEmpty()) {
            // deviceList不为空
            StringBuffer sb = new StringBuffer();
            for (UsbDevice device : deviceList.values()) {
                sb.append(device.toString());
                sb.append("\n");
                // 输出设备信息
                Log.d(TAG, "DeviceInfo: " + device.getVendorId() + ", " + device.getProductId());
                // 枚举到设备
                if (device.getVendorId() == VendorID && device.getProductId() == ProductID) {
                    usbDevice = device;
                    Log.d(TAG, "枚举设备成功");
                } else {
                    Log.d(TAG, "枚举设备失败");
                }
            }
        }
    }

    /**
     * 找设备接口
     */
    private void findInterface() {
        if (usbDevice != null) {
            Log.d(TAG, "接口数量 : " + usbDevice.getInterfaceCount());
            for (int i = 0; i < usbDevice.getInterfaceCount(); i++) {
                UsbInterface intf = usbDevice.getInterface(i);
                // 根据手上的设备做一些判断，可以在枚举到设备时打印出来
                System.out.println("*************************");
                System.out.println("getInterfaceClass = " + intf.getInterfaceClass());
                System.out.println("getInterfaceSubclass = " + intf.getInterfaceSubclass());
                System.out.println("getInterfaceProtocol = " + intf.getInterfaceProtocol());
                if (intf.getInterfaceClass() == 3 && intf.getInterfaceSubclass() == 0 && intf.getInterfaceProtocol() == 0) {
                    usbInterface = intf;
                    Log.d(TAG, "找到我的设备接口");
                    break;
                } else {
                    Log.d(TAG, "没有符合的设备接口");
                }
            }
        }
    }

    /**
     * 打开设备
     */
    private boolean openDevice() {
        if (usbInterface != null) {
            UsbDeviceConnection conn = null;
            // 在open前判断是否有连接权限；对于连接权限可以静态分配，也可以动态分配权限，可以查阅相关资料
            if (usbManager.hasPermission(usbDevice)) {
                conn = usbManager.openDevice(usbDevice);
            } else {
                Log.e(TAG, "权限不足");
            }

            if (conn == null) {
                Log.e(TAG, "打开设备失败");
                return false;
            }

            if (conn.claimInterface(usbInterface, true)) {
                // Android 设备已经连上 HID 设备
                usbDeviceConnection = conn;
                Log.d(TAG, "打开设备成功");
                return true;
            } else {
                Log.e(TAG, "无法打开连接通道");
                conn.close();
            }
        } else {
            Log.e(TAG, "USB 接口为空");
        }
        return false;
    }

    /**
     * 分配端点，IN | OUT，即输入输出；此处我直接用1为OUT端点，0为IN，当然你也可以通过判断
     */
    private void assignEndpoint() {
        Log.i(TAG, "USB接口端点数量 = " + usbInterface.getEndpointCount());
        if (usbInterface == null) {
            Log.e(TAG, "USB 端点为空");
            return;
        }
        UsbEndpoint ep = usbInterface.getEndpoint(0);
        // 开启线程接收数据
        threadDataIn = new Thread(() -> {
            while (!isInterrupted) {
                try {
                    byte[] buffer = new byte[255];
                    int transfer = usbDeviceConnection.bulkTransfer(ep, buffer, buffer.length, 1000);
                    // 发送广播
                    Intent intent = new Intent();
                    if (transfer < 0) {
                        Log.e(TAG, "bulkIn返回输入为负数 = " + transfer);
                        intent.putExtra("success", false);
                        intent.putExtra("data", transfer);
                    } else {
                        Log.i(TAG, "数据接收成功 = " + transfer + " / " + Arrays.toString(buffer));
                        intent.putExtra("success", true);
                        intent.putExtra("data", buffer);
                    }
                    intent.setAction("cn.minbb.hid.services.DataService");
                    sendBroadcast(intent);
                } catch (Exception e) {
                    e.printStackTrace();
                    break;
                }
            }
        });
        threadDataIn.start();
//        if (usbInterface.getEndpoint(1) != null) {
//            epOut = usbInterface.getEndpoint(1);
//        }
//        if (usbInterface.getEndpoint(0) != null) {
//            epIn = usbInterface.getEndpoint(0);
//        }
    }

    public void onClick(View view) {
        switch (view.getId()) {
            case R.id.startService:
                DataService.start(this);
                break;
            case R.id.stopService:
                stopService(new Intent(MainActivity.this, DataService.class));
                break;
        }
    }
}
